import random

import phonenumbers
import pytest

from phone_gen import PhoneNumber, PhoneNumberNotFound
from phone_gen.patterns import PATTERNS


@pytest.mark.phonenumbers
@pytest.mark.parametrize("count", range(15))
@pytest.mark.parametrize("country", PATTERNS["data"].keys())
def test_patterns(country: str, count: int):  # noqa: ARG001
    phone_number = PhoneNumber(country)
    number = phone_number.get_number()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize("count", range(15))
@pytest.mark.parametrize("country", PATTERNS["data"].keys())
def test_national(country: str, count: int):  # noqa: ARG001
    phone_number = PhoneNumber(country)
    number = phone_number.get_national()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize("count", range(15))
@pytest.mark.parametrize("country", PATTERNS["data"].keys())
def test_mobile(country: str, count: int):  # noqa: ARG001
    phone_number = PhoneNumber(country)
    number = phone_number.get_mobile()
    num_obj = phonenumbers.parse(number, country)
    assert phonenumbers.is_valid_number_for_region(num_obj, country)


def test_info():
    phone_number = PhoneNumber("gb")
    assert phone_number.info().startswith("libphonenumber")


def test_get_code():
    country = random.choice(tuple(PATTERNS["data"].keys()))
    phone_number = PhoneNumber(country)
    assert phone_number.get_code() == PATTERNS["data"][country]["code"]


def test_invalid_country():
    with pytest.raises(PhoneNumberNotFound):
        PhoneNumber("qwe")


def test_str_method():
    phone_number = PhoneNumber("GB")
    assert str(phone_number).startswith("<PhoneNumber(libphonenumber")



"""CLI."""

import argparse

from . import __version__
from ._phone_number import PhoneNumber, PhoneNumberNotFound

parser = argparse.ArgumentParser(
    prog="phone-gen",
    add_help=True,
    description="International phone number generation",
)
parser.add_argument(
    "-v",
    "--version",
    action="version",
    version=f"%(prog)s {__version__} ({PhoneNumber.info()})",
)
parser.add_argument(
    "country",
    help='Country code or country name. Example: "GB" or "GBR" or "Great Britain"',
    metavar="country",
    nargs="+",
)
parser.add_argument(
    "-f",
    "--not-full",
    action="store_false",
    help="Get a phone number without a country code",
    dest="full",
)
parser.add_argument(
    "-m",
    "--mobile",
    action="store_true",
    help="Get mobile phone number",
    dest="mobile",
)
parser.add_argument(
    "-n",
    "--national",
    action="store_true",
    help="Get national phone number",
    dest="national",
)


def main() -> None:
    """CLI start function."""
    args = parser.parse_args()
    country = " ".join(args.country)
    try:
        phone_number = PhoneNumber(country)
        if args.national:
            number = phone_number.get_national(args.full)
        elif args.mobile:
            number = phone_number.get_mobile(args.full)
        else:
            number = phone_number.get_number(args.full)
        print(number)  # noqa: T201
    except PhoneNumberNotFound as error:
        print(f"Error: {error.args[0]}")  # noqa: T201



import phonenumbers
import pytest

from phone_gen import PhoneNumber
from phone_gen.alt_patterns import ALT_PATTERNS


@pytest.mark.phonenumbers
@pytest.mark.parametrize("count", range(5))
@pytest.mark.parametrize(
    ("code", "ref"),
    [(key, value["ref"]) for key, value in ALT_PATTERNS.items() if "pattern" in value and "ref" in value],
)
def test_alt_pattern(code: str, ref: str, count: int):  # noqa: ARG001
    number = PhoneNumber(code).get_number()
    num_obj = phonenumbers.parse(number, code)
    assert phonenumbers.is_valid_number_for_region(num_obj, ref)


@pytest.mark.phonenumbers
@pytest.mark.parametrize("count", range(5))
@pytest.mark.parametrize(
    ("code", "ref"),
    [(key, value["ref"]) for key, value in ALT_PATTERNS.items() if "pattern" not in value and "ref" in value],
)
def test_ref(code: str, ref: str, count: int):  # noqa: ARG001
    number = PhoneNumber(code).get_number()
    num_obj = phonenumbers.parse(number, code)
    assert phonenumbers.is_valid_number_for_region(num_obj, ref)



"""
International phone number generation.

This module was created exclusively for generating test data

Support ISO 3166-2, ISO 3166-3 and Country Name
----
from phone_gen import PhoneNumber

phone_number = PhoneNumber('GB')

# Get a phone number
number = phone_number.get_number()
print(number)  # +442908124840

# Get a country code
country_code = phone_number.get_code()
print(country_code)  # 44

# Get a phone number without a country code
number = phone_number.get_number(full=False)
print(number)  # 183782623
----

Resources:
    * libphonenumber https://github.com/google/libphonenumber
"""

from ._phone_number import PhoneNumber, PhoneNumberNotFound, clean_alt_patters, load_alt_patters

try:
    from .__version__ import version as __version__
except ImportError:  # pragma: no cover
    __version__ = "unknown"

__all__ = [
    "PhoneNumber",
    "PhoneNumberNotFound",
    "__version__",
    "clean_alt_patters",
    "load_alt_patters",
]



"""Update CLI."""

import argparse
import logging
import re
import sys
from pathlib import Path
from subprocess import run as cmd
from typing import Final

import requests
from patterns_generator import get_latest, main, parsing_version

ROOT_DIR: Final[Path] = Path(__file__).absolute().parent.parent
logging.basicConfig(format="[%(asctime)s] %(levelname)s:%(message)s", level=logging.INFO)


def arg_parser() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        prog="update",
        add_help=True,
        description="Update phone-gen patterns",
    )
    parser.add_argument("-u", "--update", dest="update", help="Need to update?", action="store_true")
    parser.add_argument(
        "-p",
        "--ignore-phonenumbers",
        dest="ignore_phonenumbers",
        help="Ignore phonenumbers versions",
        action="store_true",
    )
    parser.add_argument("-b", "--no-black", dest="no_black", help="Do not run black", action="store_true")
    parser.add_argument("-f", "--no-flake", dest="no_flake", help="Do not run flake8", action="store_true")
    parser.add_argument("-t", "--no-tests", dest="no_tests", help="Do not run tests", action="store_true")
    return parser.parse_args()


def is_need_update(version: str) -> bool:
    """Is need update libphonenumber."""
    if (pattern_file := ROOT_DIR / "phone_gen" / "patterns.py").exists():
        with pattern_file.open("r") as _file:
            file = _file.read()
            if match := re.findall(r'libphonenumber v([\d.]+)"', file):
                return version != match[0]
    return True


def check_phonenumbers_libs(version: str) -> bool:
    """Check pypi phonenumbers version."""
    response = requests.get("https://pypi.org/pypi/phonenumbers/json", timeout=60)
    response.raise_for_status()
    releases = response.json()["releases"]
    return version in releases


def update_pipfile(version: str) -> None:
    """Update Pipfile file."""
    logging.info("Update Pipfile")
    if (pipfile_path := ROOT_DIR / "Pipfile").exists():
        with pipfile_path.open("r") as _file:
            pipfile = _file.read()
        pipfile = re.sub(r'phonenumbers\s?=\s?"==[\d.]+"', f'phonenumbers = "=={version}"', pipfile)
        with pipfile_path.open("w") as _file:
            _file.write(pipfile)
        return
    logging.critical("Not found Pipfile")
    sys.exit(-1)


def update_workflow(version: str) -> None:
    """Update GitHub workflow file."""
    logging.debug("Update workflow")
    if (workflow_path := ROOT_DIR / ".github" / "workflows" / "python-package.yml").exists():
        with workflow_path.open("r") as _file:
            workflow = _file.read()
        workflow = re.sub(r"phonenumbers==[\d.]+", f"phonenumbers=={version}", workflow)
        with workflow_path.open("w") as _file:
            _file.write(workflow)
        return
    logging.critical("Not found workflow file")
    sys.exit(-1)


def run(args: argparse.Namespace) -> None:
    """Run update script."""
    version = parsing_version(get_latest())
    if not args.update and not is_need_update(version=version):
        logging.info("No update required")
        sys.exit(0)
    is_phonenumbers = check_phonenumbers_libs(version=version)
    if not is_phonenumbers and not args.ignore_phonenumbers:
        logging.critical(f"No actual version phonenumbers. {version}")
        sys.exit(-1)
    if is_phonenumbers:
        update_pipfile(version=version)
        update_workflow(version=version)

    logging.info("Update requirements")
    cmd(["pipenv", "update", "-d"], cwd=ROOT_DIR, check=False)

    logging.info("Update patterns")
    main(patterns_tag=f"v{version}")

    if not args.no_black:
        logging.info("Run format")
        cmd(["pipenv", "run", "format"], cwd=ROOT_DIR, check=False)
    if not args.no_flake:
        logging.info("Run check")
        cmd(["pipenv", "run", "check"], cwd=ROOT_DIR, check=False)
    if not args.no_tests:
        logging.info("Run tests")
        cmd(["pytest", "tests/"], cwd=ROOT_DIR, check=False)
    logging.info("Done")


if __name__ == "__main__":
    run(args=arg_parser())



import random
from dataclasses import dataclass, field
from re import sub
from typing import Dict, Optional

from string_gen import StringGen

from .alt_patterns import ALT_PATTERNS
from .country_name import COUNTRY_NAME
from .iso3 import ISO3
from .patterns import PATTERNS

ALTERNATIVE_FILE: dict = {}


def clean_alt_patters() -> None:
    ALTERNATIVE_FILE.clear()


def load_alt_patters(patters: dict) -> None:
    clean_alt_patters()
    ALTERNATIVE_FILE.update(patters)


class PhoneNumberNotFound(Exception):
    """Not found country."""


@dataclass
class PhoneNumber:
    code: str
    _country: Optional[Dict[str, str]] = field(default=None, init=False, repr=False)

    def __str__(self) -> str:
        return f"<{type(self).__name__}({self.info()})>"

    def __post_init__(self) -> None:
        self.code = sub(r"\W", "", self.code).upper()
        if (country := self._find(self.code)) is None:
            raise PhoneNumberNotFound(f'Not found country "{self.code}"')
        self._country = country

    def _find(self, value: str) -> Optional[Dict[str, str]]:
        if country := ALTERNATIVE_FILE.get(value) or PATTERNS["data"].get(value):
            return country
        if alt_country := ALT_PATTERNS.get(value):
            return alt_country if "pattern" in alt_country else self._find(alt_country["ref"])
        if country_name := COUNTRY_NAME.get(value):
            return self._find(country_name["code"])
        if ico3 := ISO3.get(value):
            return self._find(ico3["code"])
        return None

    @staticmethod
    def info() -> str:
        return PATTERNS["info"]

    def get_code(self) -> str:
        return self._country["code"]

    def get_number(self, full: bool = True) -> str:
        return self.get_national(full) if random.randint(0, 1) else self.get_mobile(full)

    def get_mobile(self, full: bool = True) -> str:
        number = StringGen(self._country.get("mobile", self._country["pattern"])).render()
        return f"+{self._country['code']}{number}" if full else number

    def get_national(self, full: bool = True) -> str:
        number = StringGen(self._country["pattern"]).render()
        # Could not find problem
        if number.startswith("49") and self._country["code"] == "49":  # pragma: no cover
            return self.get_national(full)
        return f"+{self._country['code']}{number}" if full else number



import argparse
import io
import json
import re
import tarfile
import tempfile
from datetime import datetime
from pathlib import Path
from re import match
from typing import Final, Generator
from xml.etree import ElementTree

import requests

root: Final[Path] = Path(__file__).absolute().parent.parent

TEMPLATE: Final[str] = """# -*- coding: utf-8 -*-
\"""
Auto-generated file {datetime} UTC (v3).
Resource: https://github.com/google/libphonenumber {version}.
\"""


PATTERNS = {patterns}

"""
XML_FILE: Final[str] = "PhoneNumberMetadata.xml"
ARCHIVE_PATH: Final[str] = "libphonenumber-{version}/resources/{file}"
SOURCE_TAG: Final[str] = "https://github.com/google/libphonenumber/archive/{tag}.tar.gz"
DATETIME_FORMAT: Final[str] = "%Y-%m-%d %H:%M:%S"


def arg_parser() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        prog="pattern_generator",
        add_help=True,
        description="Pattern generator to phone-gen",
    )
    parser.add_argument("-t", "--tag", dest="tag", help="libphonenumber tag", default="latest")
    return parser.parse_args()


class Parser:
    def __init__(self, source: str):
        self.root = ElementTree.fromstring(source)
        self.line_tag = "fixedLine"
        self.pattern_tag = "nationalNumberPattern"
        self.mobile_tag = "mobile"

    def render(self) -> Generator[tuple[str, dict[str, str]], None, None]:
        for territory in self.root.iter("territory"):
            attrs = territory.attrib
            if (code := attrs.get("id", "1")).isdigit():
                continue
            value = {"code": attrs.get("countryCode", "")}
            for fixed_line in territory.iter(self.line_tag):
                for national_number_pattern in fixed_line.iter(self.pattern_tag):
                    value["pattern"] = re.sub(r"\s", "", national_number_pattern.text)
            for mobile_tag in territory.iter(self.mobile_tag):
                for national_number_pattern in mobile_tag.iter(self.pattern_tag):
                    value["mobile"] = re.sub(r"\s", "", national_number_pattern.text)

            yield code, value


def get_latest() -> str:
    response = requests.get("https://github.com/google/libphonenumber/releases/latest", timeout=60)
    return response.url.split("/")[-1]


def parsing_xml(file: Path) -> dict[str, dict[str, str]]:
    with file.open("rb") as _file:
        parser = Parser(_file.read().decode())
        return dict(parser.render())


def parsing_version(tag: str) -> str:
    if version := match(r".*(?P<major>\d{1,2})\.(?P<minor>\d{1,2})\.(?P<patch>\d{1,2}).*", tag):
        return "{major}.{minor}.{patch}".format_map(version.groupdict())
    raise ValueError(f"Invalid tag: {version}")


def main(patterns_tag: str) -> str:
    if tag := get_latest() if patterns_tag == "latest" else patterns_tag:
        version = parsing_version(tag)
        with tempfile.TemporaryDirectory() as tmpdir:
            response = requests.get(SOURCE_TAG.format(tag=tag), stream=True, timeout=60)
            if not response.ok:
                raise ValueError(f"Invalid tag: {tag}")
            with tarfile.open(fileobj=io.BytesIO(response.content)) as tar_file:
                archive_path = ARCHIVE_PATH.format(version=version, file=XML_FILE)
                tar_file.extract(archive_path, tmpdir)
            xml_file = Path(tmpdir, archive_path)
            if not xml_file.exists():
                raise FileNotFoundError(xml_file.absolute())
            data = parsing_xml(xml_file)
            patterns_path = root / "phone_gen" / "patterns.py"
            with patterns_path.open("wb") as _file:
                temp = TEMPLATE.format(
                    datetime=datetime.utcnow().strftime(DATETIME_FORMAT),
                    patterns=json.dumps({"info": f"libphonenumber {tag}", "data": data}, indent=4),
                    version=tag,
                )
                _file.write(temp.encode())
        return version
    return None


if __name__ == "__main__":
    args = arg_parser()
    main(patterns_tag=args.tag)



import os
import random

import phonenumbers
import pytest

from phone_gen.patterns import PATTERNS


@pytest.mark.phonenumbers
@pytest.mark.parametrize("country", random.sample(tuple(PATTERNS["data"].keys()), 20))
@pytest.mark.parametrize("arg", ("", "-n ", "-m "))
def test_get_country(capfd: pytest.CaptureFixture, country: str, arg: str):
    os.system(f"phone-gen {arg}{country}")
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, country)
    assert phonenumbers.is_valid_number_for_region(num_obg, country)


@pytest.mark.phonenumbers
@pytest.mark.parametrize("country", random.sample(tuple(PATTERNS["data"].keys()), 20))
@pytest.mark.parametrize("arg", ("", "n", "m"))
def test_get_without_country_code(capfd: pytest.CaptureFixture, country: str, arg: str):
    os.system(f"phone-gen -f{arg} {country}")
    captured = capfd.readouterr()
    code = PATTERNS["data"][country]["code"]
    num_obg = phonenumbers.parse(f"{code}{captured.out}", country)
    assert phonenumbers.is_valid_number_for_region(num_obg, country)


def test_invalid_country(capfd: pytest.CaptureFixture):
    os.system("phone-gen qwe")
    captured = capfd.readouterr()
    assert captured.out.strip() == 'Error: Not found country "QWE"'


@pytest.mark.phonenumbers
@pytest.mark.parametrize(
    ("country_name", "code"), (("Germany", "DE"), ("Panama", "PA"), ("Turkey", "TR"), ("France", "FR"))
)
@pytest.mark.parametrize("arg", ("", "-n ", "-m "))
def test_find_country_name(capfd: pytest.CaptureFixture, country_name: str, code: str, arg: str):
    os.system(f"phone-gen {arg}{country_name}")
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, code)
    assert phonenumbers.is_valid_number_for_region(num_obg, code)


@pytest.mark.phonenumbers
@pytest.mark.parametrize(("iso", "code"), (("VNM", "VN"), ("SLE", "SL"), ("MCO", "MC")))
@pytest.mark.parametrize("arg", ("", "-n ", "-m "))
def test_find_iso3(capfd: pytest.CaptureFixture, iso: str, code: str, arg: str):
    os.system(f"phone-gen {arg}{iso}")
    captured = capfd.readouterr()
    num_obg = phonenumbers.parse(captured.out, code)
    assert phonenumbers.is_valid_number_for_region(num_obg, code)



"""Sources: https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes."""

from typing import Dict

COUNTRY_NAME: Dict[str, Dict[str, str]] = {
    "AFGHANISTAN": {"name": "Afghanistan", "code": "AF"},
    "ALANDISLANDS": {"name": "Åland Islands", "code": "AX"},
    "ALBANIA": {"name": "Albania", "code": "AL"},
    "ALGERIA": {"name": "Algeria", "code": "DZ"},
    "AMERICANSAMOA": {"name": "American Samoa", "code": "AS"},
    "ANDORRA": {"name": "Andorra", "code": "AD"},
    "ANGOLA": {"name": "Angola", "code": "AO"},
    "ANGUILLA": {"name": "Anguilla", "code": "AI"},
    "ANTARCTICA": {"name": "Antarctica", "code": "AQ"},
    "ANTIGUAANDBARBUDA": {"name": "Antigua and Barbuda", "code": "AG"},
    "ARGENTINA": {"name": "Argentina", "code": "AR"},
    "ARMENIA": {"name": "Armenia", "code": "AM"},
    "ARUBA": {"name": "Aruba", "code": "AW"},
    "ASCENSIONISLAND": {"name": "Ascension Island", "code": "AC"},
    "AUSTRALIA": {"name": "Australia", "code": "AU"},
    "AUSTRIA": {"name": "Austria", "code": "AT"},
    "AZERBAIJAN": {"name": "Azerbaijan", "code": "AZ"},
    "Bahamas": {"name": "Bahamas", "code": "BS"},
    "BAHRAIN": {"name": "Bahrain", "code": "BH"},
    "BANGLADESH": {"name": "Bangladesh", "code": "BD"},
    "BARBADOS": {"name": "Barbados", "code": "BB"},
    "BELARUS": {"name": "Belarus", "code": "BY"},
    "BELGIUM": {"name": "Belgium", "code": "BE"},
    "BELIZE": {"name": "Belize", "code": "BZ"},
    "BENIN": {"name": "Benin", "code": "BJ"},
    "BERMUDA": {"name": "Bermuda", "code": "BM"},
    "BHUTAN": {"name": "Bhutan", "code": "BT"},
    "BOLIVIA": {"name": "Bolivia", "code": "BO"},
    "BONAIRE": {"name": "Bonaire", "code": "BQ"},
    "BOSNIAANDHERZEGOVINA": {"name": "Bosnia and Herzegovina", "code": "BA"},
    "BOTSWANA": {"name": "Botswana", "code": "BW"},
    "BOUVETISLAND": {"name": "Bouvet Island", "code": "BV"},
    "BRAZIL": {"name": "Brazil", "code": "BR"},
    "BRITISHINDIANOCEANTERRITORY": {"name": "British Indian Ocean Territory", "code": "IO"},
    "BRITISHVIRGINISLANDS": {"name": "British Virgin Islands", "code": "VG"},
    "BRUNEIDARUSSALAM": {"name": "Brunei Darussalam", "code": "BN"},
    "BULGARIA": {"name": "Bulgaria", "code": "BG"},
    "BURKINAFASO": {"name": "Burkina Faso", "code": "BF"},
    "BURMA": {"name": "Myanmar", "code": "MM"},
    "BURUNDI": {"name": "Burundi", "code": "BI"},
    "CABOVERDE": {"name": "Cabo Verde", "code": "CV"},
    "CAMBODIA": {"name": "Cambodia", "code": "KH"},
    "CAMEROON": {"name": "Cameroon", "code": "CM"},
    "CANADA": {"name": "Canada", "code": "CA"},
    "CAPEVERDE": {"name": "Cabo Verde", "code": "CV"},
    "CAYMANISLANDS": {"name": "Cayman Islands", "code": "KY"},
    "CENTRALAFRICANREPUBLIC": {"name": "Central African Republic", "code": "CF"},
    "CHAD": {"name": "Chad", "code": "TD"},
    "CHILE": {"name": "Chile", "code": "CL"},
    "CHINA": {"name": "China", "code": "CN"},
    "CHRISTMASISLAND": {"name": "Christmas Island", "code": "CX"},
    "COCOSISLANDS": {"name": "Cocos (Keeling) Islands", "code": "CC"},
    "COLOMBIA": {"name": "Colombia", "code": "CO"},
    "COMOROS": {"name": "Comoros", "code": "KM"},
    "CONGO": {"name": "Republic of the Congo", "code": "CG"},
    "COOKISLANDS": {"name": "Cook Islands", "code": "CK"},
    "COSTARICA": {"name": "Costa Rica", "code": "CR"},
    "COTEDIVOIRE": {"name": "Côte d'Ivoire", "code": "CI"},
    "CROATIA": {"name": "Croatia", "code": "HR"},
    "CUBA": {"name": "Cuba", "code": "CU"},
    "CURACAO": {"name": "Curaçao", "code": "CW"},
    "CURAÇAO": {"name": "Curaçao", "code": "CW"},
    "CYPRUS": {"name": "Cyprus", "code": "CY"},
    "CZECHIA": {"name": "Czech Republic", "code": "CZ"},
    "CZECHREPUBLIC": {"name": "Czech Republic", "code": "CZ"},
    "CÔTEDIVOIRE": {"name": "Côte d'Ivoire", "code": "CI"},
    "DEMOCRATICPEOPLESREPUBLICOFKOREA": {"name": "Democratic People's Republic of Korea", "code": "KP"},
    "DEMOCRATICREPUBLICOFTHECONGO": {"name": "Democratic Republic of the Congo", "code": "CD"},
    "DENMARK": {"name": "Denmark", "code": "DK"},
    "DJIBOUTI": {"name": "Djibouti", "code": "DJ"},
    "DOMINICA": {"name": "Dominica", "code": "DM"},
    "DOMINICANREPUBLIC": {"name": "Dominican Republic", "code": "DO"},
    "EASTTIMOR": {"name": "Timor-Leste", "code": "TL"},
    "ECUADOR": {"name": "Ecuador", "code": "EC"},
    "EGYPT": {"name": "Egypt", "code": "EG"},
    "ELSALVADOR": {"name": "El Salvador", "code": "SV"},
    "ENGLAND": {"name": "England", "code": "GB"},
    "EQUATORIALGUINEA": {"name": "Equatorial Guinea", "code": "GQ"},
    "ERITREA": {"name": "Eritrea", "code": "ER"},
    "ESTONIA": {"name": "Estonia", "code": "EE"},
    "ESWATINI": {"name": "Eswatini", "code": "SZ"},
    "ETHIOPIA": {"name": "Ethiopia", "code": "ET"},
    "FALKLANDISLANDS": {"name": "Falkland Islands", "code": "FK"},
    "FAROEISLANDS": {"name": "Faroe Islands", "code": "FO"},
    "FIJI": {"name": "Fiji", "code": "FJ"},
    "FINLAND": {"name": "Finland", "code": "FI"},
    "FRANCE": {"name": "France", "code": "FR"},
    "FRENCHGUIANA": {"name": "French Guiana", "code": "GF"},
    "FRENCHPOLYNESIA": {"name": "French Polynesia", "code": "PF"},
    "FRENCHSOUTHERNTERRITORIES": {"name": "French Southern Territories", "code": "TF"},
    "GABON": {"name": "Gabon", "code": "GA"},
    "GAMBIA": {"name": "Gambia", "code": "GM"},
    "GAZASTRIP": {"name": "Gaza Strip", "code": "GS"},
    "GEORGIA": {"name": "Georgia", "code": "GE"},
    "GERMANY": {"name": "Germany", "code": "DE"},
    "GHANA": {"name": "Ghana", "code": "GH"},
    "GIBRALTAR": {"name": "Gibraltar", "code": "GI"},
    "GREATBRITAIN": {"name": "United Kingdom", "code": "GB"},
    "GREECE": {"name": "Greece", "code": "GR"},
    "GREENLAND": {"name": "Greenland", "code": "GL"},
    "GRENADA": {"name": "Grenada", "code": "GD"},
    "GUADELOUPE": {"name": "Guadeloupe", "code": "GP"},
    "GUAM": {"name": "Guam", "code": "GU"},
    "GUATEMALA": {"name": "Guatemala", "code": "GT"},
    "GUERNSEY": {"name": "Guernsey", "code": "GG"},
    "GUINEA": {"name": "Guinea", "code": "GN"},
    "GUINEABISSAU": {"name": "Guinea-Bissau", "code": "GW"},
    "GUYANA": {"name": "Guyana", "code": "GY"},
    "HAITI": {"name": "Haiti", "code": "HT"},
    "HEARDISLANDANDMCDONALDISLANDS": {"name": "Heard Island and McDonald Islands", "code": "HM"},
    "HOLYSEE": {"name": "Holy See", "code": "VA"},
    "HONDURAS": {"name": "Honduras", "code": "HN"},
    "HONGKONG": {"name": "Hong Kong", "code": "HK"},
    "HUNGARY": {"name": "Hungary", "code": "HU"},
    "ICELAND": {"name": "Iceland", "code": "IS"},
    "INDIA": {"name": "India", "code": "IN"},
    "INDONESIA": {"name": "Indonesia", "code": "ID"},
    "IRAN": {"name": "Iran", "code": "IR"},
    "IRAQ": {"name": "Iraq", "code": "IQ"},
    "IRELAND": {"name": "Ireland", "code": "IE"},
    "ISLEOFMAN": {"name": "Isle of Man", "code": "IM"},
    "ISRAEL": {"name": "Israel", "code": "IL"},
    "ITALY": {"name": "Italy", "code": "IT"},
    "JAMAICA": {"name": "Jamaica", "code": "JM"},
    "JANMAYEN": {"name": "Jan Mayen", "code": "SJ"},
    "JAPAN": {"name": "Japan", "code": "JP"},
    "JERSEY": {"name": "Jersey", "code": "JE"},
    "JORDAN": {"name": "Jordan", "code": "JO"},
    "KAZAKHSTAN": {"name": "Kazakhstan", "code": "KZ"},
    "KEELINGISLANDS": {"name": "Cocos (Keeling) Islands", "code": "CC"},
    "KENYA": {"name": "Kenya", "code": "KE"},
    "KIRIBATI": {"name": "Kiribati", "code": "KI"},
    "KOREA": {"name": "Republic of Korea", "code": "KR"},
    "KOSOVO": {"name": "Kosovo", "code": "XK"},
    "KUWAIT": {"name": "Kuwait", "code": "KW"},
    "KYRGYZSTAN": {"name": "Kyrgyzstan", "code": "KG"},
    "LAOPEOPLESDEMOCRATICREPUBLIC": {"name": "Lao People's Democratic Republic", "code": "LA"},
    "LAOS": {"name": "Lao People's Democratic Republic", "code": "LA"},
    "LATVIA": {"name": "Latvia", "code": "LV"},
    "LEBANON": {"name": "Lebanon", "code": "LB"},
    "LESOTHO": {"name": "Lesotho", "code": "LS"},
    "LIBERIA": {"name": "Liberia", "code": "LR"},
    "LIBYA": {"name": "Libya", "code": "LY"},
    "LIECHTENSTEIN": {"name": "Liechtenstein", "code": "LI"},
    "LITHUANIA": {"name": "Lithuania", "code": "LT"},
    "LUXEMBOURG": {"name": "Luxembourg", "code": "LU"},
    "MACAO": {"name": "Macao", "code": "MO"},
    "MACEDONIA": {"name": "North Macedonia", "code": "MK"},
    "MADAGASCAR": {"name": "Madagascar", "code": "MG"},
    "MALAWI": {"name": "Malawi", "code": "MW"},
    "MALAYSIA": {"name": "Malaysia", "code": "MY"},
    "MALDIVES": {"name": "Maldives", "code": "MV"},
    "MALI": {"name": "Mali", "code": "ML"},
    "MALTA": {"name": "Malta", "code": "MT"},
    "MARSHALLISLANDS": {"name": "Marshall Islands", "code": "MH"},
    "MARTINIQUE": {"name": "Martinique", "code": "MQ"},
    "MAURITANIA": {"name": "Mauritania", "code": "MR"},
    "MAURITIUS": {"name": "Mauritius", "code": "MU"},
    "MAYOTTE": {"name": "Mayotte", "code": "YT"},
    "MEXICO": {"name": "Mexico", "code": "MX"},
    "MICRONESIA": {"name": "Micronesia", "code": "FM"},
    "MIDWAYISLANDS": {"name": "Midway Islands", "code": "MI"},
    "MINOROUTLYINGISLANDS": {"name": "United States Minor Outlying Islands", "code": "UM"},
    "MOLDOVA": {"name": "Moldova", "code": "MD"},
    "MONACO": {"name": "Monaco", "code": "MC"},
    "MONGOLIA": {"name": "Mongolia", "code": "MN"},
    "MONTENEGRO": {"name": "Montenegro", "code": "ME"},
    "MONTSERRAT": {"name": "Montserrat", "code": "MS"},
    "MOROCCO": {"name": "Morocco", "code": "MA"},
    "MOZAMBIQUE": {"name": "Mozambique", "code": "MZ"},
    "MYANMAR": {"name": "Myanmar", "code": "MM"},
    "NAMIBIA": {"name": "Namibia", "code": "NA"},
    "NAURU": {"name": "Nauru", "code": "NR"},
    "NEPAL": {"name": "Nepal", "code": "NP"},
    "NETHERLANDS": {"name": "Netherlands", "code": "NL"},
    "NETHERLANDSANTILLES": {"name": "Netherlands Antilles", "code": "AN"},
    "NEWCALEDONIA": {"name": "New Caledonia", "code": "NC"},
    "NEWZEALAND": {"name": "New Zealand", "code": "NZ"},
    "NICARAGUA": {"name": "Nicaragua", "code": "NI"},
    "NIGER": {"name": "Niger", "code": "NE"},
    "NIGERIA": {"name": "Nigeria", "code": "NG"},
    "NIUE": {"name": "Niue", "code": "NU"},
    "NORFOLKISLAND": {"name": "Norfolk Island", "code": "NF"},
    "NORTHERNCYPRUS": {"name": "Northern Cyprus", "code": "NY"},
    "NORTHERNIRELAND": {"name": "Northern Ireland", "code": "GB"},
    "NORTHERNMARIANAISLANDS": {"name": "Northern Mariana Islands", "code": "MP"},
    "NORTHKOREA": {"name": "Democratic People's Republic of Korea", "code": "KP"},
    "NORTHMACEDONIA": {"name": "North Macedonia", "code": "MK"},
    "NORWAY": {"name": "Norway", "code": "NO"},
    "OMAN": {"name": "Oman", "code": "OM"},
    "PAKISTAN": {"name": "Pakistan", "code": "PK"},
    "PALAU": {"name": "Palau", "code": "PW"},
    "PALESTINE": {"name": "Palestine", "code": "PS"},
    "PANAMA": {"name": "Panama", "code": "PA"},
    "PAPUANEWGUINEA": {"name": "Papua New Guinea", "code": "PG"},
    "PARAGUAY": {"name": "Paraguay", "code": "PY"},
    "PERU": {"name": "Peru", "code": "PE"},
    "PHILIPPINES": {"name": "Philippines", "code": "PH"},
    "PITCAIRN": {"name": "Pitcairn", "code": "PN"},
    "POLAND": {"name": "Poland", "code": "PL"},
    "PORTUGAL": {"name": "Portugal", "code": "PT"},
    "PUERTORICO": {"name": "Puerto Rico", "code": "PR"},
    "QATAR": {"name": "Qatar", "code": "QA"},
    "REPUBLICOFKOREA": {"name": "Republic of Korea", "code": "KR"},
    "REPUBLICOFTHECONGO": {"name": "Republic of the Congo", "code": "CG"},
    "REUNION": {"name": "Réunion", "code": "RE"},
    "ROMANIA": {"name": "Romania", "code": "RO"},
    "RUSSIA": {"name": "Russian Federation", "code": "RU"},
    "RUSSIANFEDERATION": {"name": "Russian Federation", "code": "RU"},
    "RWANDA": {"name": "Rwanda", "code": "RW"},
    "RÉUNION": {"name": "Réunion", "code": "RE"},
    "SABA": {"name": "Saba", "code": "BQ"},
    "SAINTBARTHELEMY": {"name": "Saint Barthélemy", "code": "BL"},
    "SAINTBARTHÉLEMY": {"name": "Saint Barthélemy", "code": "BL"},
    "SAINTHELENA": {"name": "Saint Helena", "code": "SH"},
    "SAINTKITTSANDNEVIS": {"name": "Saint Kitts and Nevis", "code": "KN"},
    "SAINTLUCIA": {"name": "Saint Lucia", "code": "LC"},
    "SAINTMARTIN": {"name": "Saint Martin", "code": "MF"},
    "SAINTPIERREANDMIQUELON": {"name": "Saint Pierre and Miquelon", "code": "PM"},
    "SAINTVINCENTANDTHEGRENADINES": {"name": "Saint Vincent and the Grenadines", "code": "VC"},
    "SALVADOR": {"name": "El Salvador", "code": "SV"},
    "SAMOA": {"name": "Samoa", "code": "WS"},
    "SANMARINO": {"name": "San Marino", "code": "SM"},
    "SAOTOMEANDPRINCIPE": {"name": "Sao Tome and Principe", "code": "ST"},
    "SAUDIARABIA": {"name": "Saudi Arabia", "code": "SA"},
    "SENEGAL": {"name": "Senegal", "code": "SN"},
    "SERBIA": {"name": "Serbia", "code": "RS"},
    "SEYCHELLES": {"name": "Seychelles", "code": "SC"},
    "SIERRALEONE": {"name": "Sierra Leone", "code": "SL"},
    "SINGAPORE": {"name": "Singapore", "code": "SG"},
    "SINTEUSTATIUS": {"name": "Sint Eustatius", "code": "BQ"},
    "SINTMAARTEN": {"name": "Sint Maarten", "code": "SX"},
    "SLOVAKIA": {"name": "Slovakia", "code": "SK"},
    "SLOVENIA": {"name": "Slovenia", "code": "SI"},
    "SOLOMONISLANDS": {"name": "Solomon Islands", "code": "SB"},
    "SOMALIA": {"name": "Somalia", "code": "SO"},
    "SOUTHAFRICA": {"name": "South Africa", "code": "ZA"},
    "SOUTHGEORGIAANDTHESOUTHSANDWICHISLANDS": {"name": "South Georgia and the South Sandwich Islands", "code": "GS"},
    "SOUTHKOREA": {"name": "Republic of Korea", "code": "KR"},
    "SOUTHSUDAN": {"name": "South Sudan", "code": "SS"},
    "SPAIN": {"name": "Spain", "code": "ES"},
    "SRILANKA": {"name": "Sri Lanka", "code": "LK"},
    "SUDAN": {"name": "Sudan", "code": "SD"},
    "SURINAME": {"name": "Suriname", "code": "SR"},
    "SVALBARD": {"name": "Svalbard", "code": "SJ"},
    "SWEDEN": {"name": "Sweden", "code": "SE"},
    "SWITZERLAND": {"name": "Switzerland", "code": "CH"},
    "SYRIA": {"name": "Syrian Arab Republic", "code": "SY"},
    "SYRIANARABREPUBLIC": {"name": "Syrian Arab Republic", "code": "SY"},
    "TAIWAN": {"name": "Taiwan", "code": "TW"},
    "TAJIKISTAN": {"name": "Tajikistan", "code": "TJ"},
    "TANZANIA": {"name": "Tanzania", "code": "TZ"},
    "THAILAND": {"name": "Thailand", "code": "TH"},
    "TIMORLESTE": {"name": "Timor-Leste", "code": "TL"},
    "TOGO": {"name": "Togo", "code": "TG"},
    "TOKELAU": {"name": "Tokelau", "code": "TK"},
    "TONGA": {"name": "Tonga", "code": "TO"},
    "TRINIDADANDTOBAGO": {"name": "Trinidad and Tobago", "code": "TT"},
    "TRISTANDACUNHA": {"name": "Tristan da Cunha", "code": "TA"},
    "TUNISIA": {"name": "Tunisia", "code": "TN"},
    "TURKEY": {"name": "Turkey", "code": "TR"},
    "TURKMENISTAN": {"name": "Turkmenistan", "code": "TM"},
    "TURKSANDCAICOSISLANDS": {"name": "Turks and Caicos Islands", "code": "TC"},
    "TUVALU": {"name": "Tuvalu", "code": "TV"},
    "UGANDA": {"name": "Uganda", "code": "UG"},
    "UKRAINE": {"name": "Ukraine", "code": "UA"},
    "UNITEDARABEMIRATES": {"name": "United Arab Emirates", "code": "AE"},
    "UNITEDKINGDOM": {"name": "United Kingdom", "code": "GB"},
    "UNITEDSTATE": {"name": "United States of America", "code": "US"},
    "UNITEDSTATESMINOROUTLYINGISLANDS": {"name": "United States Minor Outlying Islands", "code": "UM"},
    "UNITEDSTATESOFAMERICA": {"name": "United States of America", "code": "US"},
    "UNITEDSTATESVIRGINISLANDS": {"name": "United States Virgin Islands", "code": "VI"},
    "URUGUAY": {"name": "Uruguay", "code": "UY"},
    "USA": {"name": "United States of America", "code": "US"},
    "USVIRGINISLANDS": {"name": "US Virgin Islands", "code": "VI"},
    "UZBEKISTAN": {"name": "Uzbekistan", "code": "UZ"},
    "VANUATU": {"name": "Vanuatu", "code": "VU"},
    "VENEZUELA": {"name": "Venezuela", "code": "VE"},
    "VIETNAM": {"name": "Vietnam", "code": "VN"},
    "WALES": {"name": "Wales", "code": "GB"},
    "WALLISANDFUTUNA": {"name": "Wallis and Futuna", "code": "WF"},
    "WESTBANK": {"name": "West Bank", "code": "WB"},
    "WESTERNSAHARA": {"name": "Western Sahara", "code": "EH"},
    "YEMEN": {"name": "Yemen", "code": "YE"},
    "ZAMBIA": {"name": "Zambia", "code": "ZM"},
    "ZIMBABWE": {"name": "Zimbabwe", "code": "ZW"},
}



import pytest

from phone_gen.country_name import COUNTRY_NAME
from phone_gen.iso3 import ISO3
from phone_gen.patterns import PATTERNS


@pytest.mark.parametrize("country", PATTERNS["data"].keys())
def test_ico3(country: str):
    assert [i for i in ISO3.values() if i["code"] == country]


@pytest.mark.parametrize("country", PATTERNS["data"].keys())
def test_country_name(country: str):
    assert [i for i in COUNTRY_NAME.values() if i["code"] == country]



import random
from typing import Iterator

import phonenumbers
import pytest

from phone_gen import PhoneNumber, clean_alt_patters, load_alt_patters
from phone_gen.patterns import PATTERNS


@pytest.fixture(params=random.sample(tuple(PATTERNS["data"].keys()), 20))
def load_fixture(request: pytest.FixtureRequest) -> Iterator[str]:
    load_alt_patters({request.param: {"code": "0", "pattern": "[2]{7}", "mobile": "[1]{9}"}})
    yield request.param
    clean_alt_patters()


@pytest.mark.phonenumbers
def test_load_alt_patterns(load_fixture: str):
    phone_number = PhoneNumber(load_fixture)
    assert phone_number.get_national() == "+0{}".format("2" * 7)
    assert phone_number.get_mobile() == "+0{}".format("1" * 9)
    clean_alt_patters()
    phone_number = PhoneNumber(load_fixture)
    num_obj = phonenumbers.parse(phone_number.get_national(), load_fixture)
    assert phonenumbers.is_valid_number_for_region(num_obj, load_fixture)
    num_obj = phonenumbers.parse(phone_number.get_mobile(), load_fixture)
    assert phonenumbers.is_valid_number_for_region(num_obj, load_fixture)
